@@ -419,6 +419,8 @@ gnome_entry_history_changed (GConfClient* client,
 {
 	GnomeEntry *gentry;
 
+	GDK_THREADS_ENTER();
+
 	gentry = GNOME_ENTRY (user_data);
 
 	/* If we're getting a notification from saving our own
@@ -427,10 +429,13 @@ gnome_entry_history_changed (GConfClient* client,
 	if (gentry->_priv->saving_history) {
 		gentry->_priv->saving_history = FALSE;
 
-		return;
+		goto end;
 	}
 
 	gnome_entry_load_history (gentry);
+
+ end:
+	GDK_THREADS_LEAVE();
 }
 
 /* FIXME: Make this static */
@@ -591,6 +596,10 @@ set_combo_items (GnomeEntry *gentry)
 
 		g_list_free (strings);
 	}
+	else
+	{
+		gtk_list_clear_items (GTK_LIST (GTK_COMBO (gentry)->list), 0, -1);
+	}
 
 	/* Restore the text in the entry and clear our changed flag. */
 
@@ -813,6 +822,7 @@ gnome_entry_save_history (GnomeEntry *gentry)
 	gentry->_priv->saving_history = TRUE;
 	gconf_client_set_list (gentry->_priv->gconf_client, key, GCONF_VALUE_STRING, gconf_items, NULL);
 
+	g_slist_free (gconf_items);
 	g_free (key);
 }
 
